#include "NGTP-IF1-DispatchingServices.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"

#include <stdio.h>
#include <stdlib.h>

int main (int argc, char** argv)
{
   const OSOCTET* msgptr;
   OSBOOL     aligned = TRUE;
   OSBOOL     trace = TRUE;
   OSBOOL     verbose = FALSE;
   FILE*      fp;
   char*      filename = "message.dat";
   int        i, len, stat;

   /* Process Command line arguments */
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [-a | -u] [-v] "
                    "[-o <filename>] [-notrace]\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 1;
         }
      }
   }

   // Create an instance of the compiler generated class.
   // This example uses a dynamic message buffer..
   ASN1PEREncodeBuffer encodeBuffer (aligned);
   ASN1T_DispatchingUplinkMessage data;
   ASN1C_DispatchingUplinkMessage uplinkPDU (encodeBuffer, data);

   // Populate structure of generated type
   data.protocolVersion = 100;
   data.serviceType = 22;

   const OSOCTET testData[] = {
      0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88
   } ;
   data.nodeId.numocts = sizeof(testData);
   OSCRTLMEMCPY (data.nodeId.data, testData, sizeof(testData));

   data.location.latitude = 57123456;
   data.location.longitude = 10123456;

   ASN1T_NetworkInfo_cellular cellular;
   cellular.mccNetwork = "111";
   cellular.mncNetwork = "234";
   cellular.mccSim = "777";
   cellular.mncSim = "765";

   data.networkInfo.t = T_NetworkInfo_cellular;
   data.networkInfo.u.cellular = &cellular;

   data.provisioningVersion.defaultConfigurationId = 1023;
   data.provisioningVersion.currentConfigurationId = 0;

   data.dispatchingId.initiator = DispatchingIdentifier_initiator::tu;
   data.dispatchingId.dispatchingSession = 1011;

   data.tuId = "TU identifier";
   data.language = "aaa";

   const OSOCTET testData2[] = {
      0xff, 0xee, 0xee, 0xdd
   } ;
   data.payload.numocts = sizeof(testData2);
   data.payload.data = testData2;

   // Encode 
   encodeBuffer.setTrace (trace);

   if ((stat = uplinkPDU.Encode ()) == 0)
   {
      if (trace) {
         printf ("Encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         encodeBuffer.hexDump ();
         printf ("Binary dump:\n");
         encodeBuffer.binDump ("Data");
      }
      msgptr = encodeBuffer.getMsgPtr ();
      len = encodeBuffer.getMsgLen ();
   }
   else
   {
      printf ("Encoding failed\n");
      encodeBuffer.printErrorInfo ();
      return (-1);
   }

   // Write the encoded message out to the output file
   fp = fopen (filename, "wb");
   if (0 != fp) {
      fwrite (msgptr, 1, len, fp);
      fclose (fp);
   }
   else {
      printf ("Error opening %s for write access\n", filename);
      return -1;
   }

   return 0;
}
